<?php
/**
 * Plugin Name:       Martins Free Backlink Linkbuilding Network
 * Description:       100% free, automatic and effective SEO optimizing backlink linkbuilding network.
 * Version:           1.1.2
 * Requires at least: 4.7
 * Requires PHP:      7.0
 * Author:            Bundgaard
 * Author URI:        https://free-backlink-network.sjuup.dk
 * License:           GPLv2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       martinsLinkNetwork
 * Domain Path:       /languages
 */

if(!defined ('ABSPATH')) {die;} // Block direct access to the file


class martinsLinkNetworkFront 
{
    private $version = "1.1.2";
    private $cacheFile = WP_CONTENT_DIR . "/uploads/martinsLinkNetworkCache.txt";
    private $logFile = WP_CONTENT_DIR . "/uploads/martinsLinkNetworkLog.txt";
    private $links = [];
    private $data = "";
    private $usedKeyword = [];
    
    
    function __construct() {
        add_action('wp', [$this, "getData"], 1000000);
        add_filter('the_content', [$this, "insertLinksContent"], 1000000);
        add_action('wp_footer', [$this, "insertLinksFooter"], 1000000);
    }
     
    
    public function getData() 
    {
        // Initiate file system
        require_once (ABSPATH . "/wp-admin/includes/file.php");
        WP_Filesystem();
        //global $wp;
        global $wp_filesystem;
        
        // Get log data
        $logData = "";
        if ($wp_filesystem->exists($this->logFile)) {
            $logData = $wp_filesystem->get_contents($this->logFile);
        }
        
        // Get data from cache if exists
        $isCache = false;        
        if ($wp_filesystem->exists($this->cacheFile) && time() - $wp_filesystem->mtime($this->cacheFile) < 24 * 3600) { // 1 day
            $this->data = $wp_filesystem->get_contents($this->cacheFile);
            $isCache = true;
        }
        
        // Get data from API if no cached data
        else {
            $result = wp_remote_post("https://linknetwork.martinstools.com/api/domains", ['timeout' => 30, 'method' => 'POST', 'body' => ["url" => get_site_url(), "name" => get_bloginfo("name"), "description" => get_bloginfo("description"), "version" => $this->version, "logData" => $logData]]);
            $this->data = $result["body"];
            
            if ($this->data <> "") {
                if (!$wp_filesystem->put_contents($this->cacheFile, $this->data)) {
                    echo("<hr style='margin:0;padding:0;height:1px;border:0;' /><div style='text-align:center;'><b>Martins Link Network plugin, could not write cache to upload folder!</b><br />Please check your folder permissions...</div>");
                }
            }
        }
        
        // Save current url to log
        if (!$isCache) {
            // Cleanup log and save only this url if new data was requested from server
            $log = [md5(get_permalink()) => get_permalink()];
        }
        else {
            $log = json_decode($logData, true);
            $log[md5(get_permalink())] = get_permalink();
        }
        $wp_filesystem->put_contents($this->logFile, json_encode($log));       
       
        // Extract links from data
        if ($this->data <> "") {
            $this->links = json_decode($this->data, true);
        }      
    }
    
    
    function insertLinksContent($content) 
    {
        // Check if we're inside the main loop in a single Post.
        if (count($this->links) > 0 && is_singular() && in_the_loop() && is_main_query()) {
            $i = 0;
            foreach($this->links as $link) {
                foreach($link["keywords"] as $keyword) {
                    $pos = strrpos($content, " " . $keyword . " ");
                    if($pos !== false && !in_array($keyword, $this->usedKeyword))
                    {
                        $content = substr_replace($content, "<a href='" . $link["scheme"] . "://" . $link["domain"] . "' target='_blank'>" . $keyword . "</a>", $pos+1, strlen($keyword));
                        unset($this->links[$i]);
                        $this->usedKeyword[] = $keyword;
                        break; // Only make 1 keyword clickable
                    }
                }
                
                $i++;
            }
            
            return $content;
        }

        return $content;
    }
    
    
    public function insertLinksFooter() 
    {
        // Show links
        if (count($this->links) > 0) {  
            $linkStr = "";
            foreach($this->links as $link) {
                if ($link["name"] == "") {
                    $link["name"] = $link["domain"];
                }

                $linkStr .= "<a href='" . $link["scheme"] . "://" . $link["domain"] . "'>" . $link["name"] . "</a> - ";
            }
            
            // Build allowed html array
            $allowedHtml = wp_kses_allowed_html();
            $allowedHtml['a'] = array();
            $allowedHtml['a']['href'] = array();
            
            echo("<div><hr style='margin:0;padding:0;height:1px;border:0;' /><div style='padding:5px;text-align:center;font-size:11px;' />" . wp_kses(substr($linkStr, 0, -3), $allowedHtml) . "</div></div>");
        }
    }
    
}


class martinsLinkNetworkAdmin 
{
    function __construct() {
        add_action('admin_menu', [$this, "addMenuItems"]);
    }
    
    
    function addMenuItems() {
        global $submenu;
        
        $url = wp_parse_url(get_site_url());
        $submenu['index.php'][] = array("Martins linkbuilding", "manage_options" , "https://linknetwork.martinstools.com/admin/#/statswp/" . $url["host"] ); 
    }  
}


// Start the show
if (!is_admin()) {
    $martinsLinkNetworkFront = new martinsLinkNetworkFront();   
}
else {
    $martinsLinkNetworkAdmin = new martinsLinkNetworkAdmin();
}